import asyncio
import os
from pathlib import Path

import click
import github.GithubException
from dotenv import load_dotenv
from github import Github, Auth
from github.Repository import Repository
from rich import print

from src.util import logger


@click.command()
@click.option("--verbose", "-v", is_flag=True, help="Print all the output.")
@click.option("--dry-run", "-d", is_flag=True, help="Run through the process, but make no changes to GitHub.")
@click.argument("filename", type=click.Path(exists=True, readable=True), default=".env")
def cli(verbose: bool, dry_run: bool, filename: click.Path):
    """
    Update GitHub repository secrets and environment variables from a .env file. Requires the values for `GH_REPO`, `GH_ACCOUNT`, and `GH_TOKEN` in your .env file. This script finds sensitive key/value pairs in FILENAME (by looking for substrins like KEY, TOKEN, SECRET, etc. in the key name) and updates the GitHub repository secrets with them. It then updates the GitHub repository environment variables with the remaining key/value pairs.
    """

    if dry_run:
        logger.info("Dry run mode enabled. No changes will be made to GitHub.")
    results = asyncio.run(perform_update(filename, verbose, dry_run))
    if verbose:
        print(results)
    total_secrets = len(results["SECRETS"])
    total_vars = len(results["VARIABLES"])
    print(f"Updated {total_secrets} secrets and {total_vars} variables.")


async def perform_update(filename: click.Path, verbose: bool = False, dry_run: bool = False):
    """
    Performs the update of the GitHub repository secrets and environment variables.
    :param filename: file to use for secrets, variables, and github repo
    :return:
    """
    if not filename:
        raise ValueError("No filename specified. Exiting to avoid an accident.")

    load_dotenv(str(filename))
    github_repo = os.environ.get("GH_REPO", None)
    github_account = os.environ.get("GH_ACCOUNT", None)

    file_path = Path(filename.__str__())
    # Read the .env file and convert it to a JSON object
    secrets = {}
    not_secrets = {}
    SECRETS_INDICATORS = ["secret", "key", "token", "bearer", "password", "pass", "pwd", "pword", "hash"]
    logger.debug(f"Reading file {file_path}")

    with file_path.open("r") as file:
        for line in file:
            line = line.strip()
            if not line or line.startswith("#") or line.startswith(";") or "=" not in line:
                continue
            key, value = line.strip().split("=", 1)

            match key:
                case key if key.startswith("AWS_PROFILE"):
                    continue
                case key if any(indicator in key.casefold() for indicator in SECRETS_INDICATORS):
                    secrets[key] = value
                case _:
                    not_secrets[key] = value

    try:
        repo = get_github_repo(github_account, github_repo)
    except github.GithubException as e:
        logger.critical("You must add GH_REPO to your env file.")
        exit(1)

    secret_update_result = await create_or_update_github_secrets(repo=repo, env_data=secrets, dry_run=dry_run)
    not_secret_update_result = await create_or_update_github_variables(repo=repo, env_data=not_secrets, dry_run=dry_run)

    results = {"SECRETS": secret_update_result, "VARIABLES": not_secret_update_result}

    return results


def get_github_repo(github_account, github_repo_name) -> Repository:
    """
    Get the GitHub repository object.
    :param github_account: Name of your GitHub account (i.e. your username), or the name of the organization.
    :param github_repo_name: Name of the repository.
    :return: a GitHub repository object
    """
    auth = Auth.Token(os.environ.get("GH_TOKEN", None))
    g = Github(auth=auth)
    try:
        repo = g.get_user(github_account).get_repo(github_repo_name)
    except github.GithubException.UnknownObjectException as e:
        logger.critical(e)
        try:
            repo = g.get_organization(github_account).get_repo(github_repo_name)
        except github.GithubException.UnknownObjectException as e:
            logger.critical("You must add GH_USER to your env file.")
            raise e

    return repo


async def create_or_update_github_secrets(repo, env_data, dry_run: bool=False):
    """
    Create or update GitHub repository secrets.
    :param dry_run: run through the process, but make no changes to GitHub
    :param repo: the repository on which to operate
    :param env_data: the secrets and their values in a dictionary
    :return: list of results
    """
    secrets = await asyncio.to_thread(repo.get_secrets)
    secret_names = [secret.name for secret in secrets]
    dry_run_prefix = "[DRY RUN] " if dry_run else ""

    tasks = []
    for env_var_name, env_var_value in env_data.items():
        # Create or update secrets
        if env_var_name in secret_names:
            logger.info(f"{dry_run_prefix}Updating secret {env_var_name}...")
            tasks.append(asyncio.to_thread(repo.create_secret, env_var_name, env_var_value))
        else:
            logger.info(f"{dry_run_prefix}Creating secret {env_var_name}...")
            tasks.append(asyncio.to_thread(repo.create_secret, env_var_name, env_var_value))

    for secret_name in secret_names:
        if secret_name not in env_data.keys():
            logger.info(f"{dry_run_prefix}Deleting secret {secret_name}...")
            tasks.append(asyncio.to_thread(repo.delete_secret, secret_name))

    if dry_run:
        results = [secret for secret in secrets]
    else:
        results = await asyncio.gather(*tasks)
    return results


async def create_or_update_github_variables(repo, env_data, dry_run: bool = False):
    """
    Create or update GitHub repository environment variables.
    :param repo: the repository on which to operate
    :param dry_run: run through the process, but make no changes to GitHub
    :param env_data: the secrets and their values in a dictionary
    :return: list of results
    """
    vars = await asyncio.to_thread(repo.get_variables)
    var_names = [var.name for var in vars]
    dry_run_prefix = "[DRY RUN] " if dry_run else ""

    tasks = []
    for env_var_name, env_var_value in env_data.items():
        # Create or update secrets
        if env_var_name in var_names:
            logger.info(f"{dry_run_prefix}Updating variable {env_var_name}...")

            def delete_then_create_variable(repo, env_var_name, env_var_value):
                repo.delete_variable(env_var_name)
                return repo.create_variable(env_var_name, env_var_value)

            # tasks.append(asyncio.to_thread(repo.delete_variable, env_var_name))
            tasks.append(asyncio.to_thread(delete_then_create_variable, repo, env_var_name, env_var_value))
        else:
            logger.info(f"{dry_run_prefix}Creating variable {env_var_name}...")
            tasks.append(asyncio.to_thread(repo.create_variable, env_var_name, env_var_value))

    for var_name in var_names:
        if var_name not in env_data.keys():
            logger.info(f"{dry_run_prefix}Deleting variable {var_name}...")
            tasks.append(asyncio.to_thread(repo.delete_variable, var_name))

    if dry_run:
        results = [var for var in vars]
    else:
        results = await asyncio.gather(*tasks)

    return results


if __name__ == "__main__":
    cli()




import boto3


def list_ec2_resources():
    ec2 = boto3.resource("ec2")

    print("Listing EC2 Instances:")
    for instance in ec2.instances.all():
        print(instance.id, instance.state, instance.instance_type)

    print("\nListing EBS Snapshots:")
    ec2_client = boto3.client("ec2")
    snapshots = ec2_client.describe_snapshots(OwnerIds=["self"])
    for snapshot in snapshots["Snapshots"]:
        print(snapshot["SnapshotId"], snapshot["VolumeId"], snapshot["VolumeSize"])

    print("\nListing Elastic IPs:")
    elastic_ips = ec2_client.describe_addresses()
    for eip in elastic_ips["Addresses"]:
        print(eip["PublicIp"], eip.get("InstanceId", "Unassociated"))


if __name__ == "__main__":
    list_ec2_resources()



import json
import os
import click
import boto3
from botocore.exceptions import ClientError
from dotenv import load_dotenv

APP_NAME = os.getenv("APP_NAME", "augint-tools")

def get_policy_arn(policy_name, iam, verbose=False, dry_run=False):
    try:
        response = iam.list_policies(Scope="Local")
        policies = response.get("Policies", [])
        for policy in policies:
            if policy["PolicyName"] == policy_name:
                return policy["Arn"]
        return None
    except ClientError as e:
        print(f"Error getting policy ARN: {e}")
        return None


def create_or_update_policy(policy_name, policy_document, iam, verbose=False, dry_run=False):
    policy_arn = get_policy_arn(policy_name, iam)
    if policy_arn:
        print(f"Updating policy: {policy_name}")
        if not dry_run:
            iam.create_policy_version(PolicyArn=policy_arn, PolicyDocument=json.dumps(policy_document), SetAsDefault=True)
    else:
        print(f"Creating policy: {policy_name}")
        if not dry_run:
            iam.create_policy(PolicyName=policy_name, PolicyDocument=json.dumps(policy_document))


def attach_policy(role_arn, policy_name, iam, verbose=False, dry_run=False):
    policy_arn = get_policy_arn(policy_name, iam)
    if not policy_arn:
        print(f"Policy {policy_name} does not exist.")
        return

    try:
        if not dry_run:
            iam.attach_role_policy(RoleName=role_arn.split("/")[-1], PolicyArn=policy_arn)
        print(f"Policy {policy_name} attached to role {role_arn}")
    except ClientError as e:
        print(f"Error attaching policy: {e}")


@click.command()
@click.option("--dev", "-D", help="Use dev env vars", is_flag=True)
@click.option("--prod", "-P", help="Use prod env vars", is_flag=True)
@click.option("--env-file", "-e", help="Path to .env file", type=click.Path(exists=True))
@click.option("--verbose", "-v", help="Verbose output", is_flag=True)
@click.option("--dry-run", "-d", help="Dry run", is_flag=True)
@click.option("--role", "-r", help="Pipeline ole to augment with the new policy", type=click.STRING)
@click.option("--stack", "-s", help="Permissions for the development or production stack", type=click.STRING)
@click.option("--aws-profile", "-a", help="AWS profile name to use", type=click.STRING)
@click.option(
    "--policy-name-prefix", "-p", help="Permissions for the development or production stack", type=click.STRING, default=APP_NAME
)
def main(dev: bool,prod: bool, env_file: click.Path, verbose: bool, dry_run: bool, role: str, stack: str, aws_profile: str, policy_name_prefix: str):
    """Attach or update a policy for a given IAM role."""

    runs = []
    if not env_file:
        load_dotenv(".env")
    else:
        load_dotenv(env_file)

    if stack and role and policy_name_prefix:
        policy_name = f"{policy_name_prefix}-pipeline-policy{stack}"
        runs.append((stack, role, policy_name))
    if dev:
        stack = os.environ["DEV_INFRA_STACK_NAME"]
        role = os.environ["TESTING_PIPELINE_EXECUTION_ROLE"]
        policy_name = f"{APP_NAME}-pipeline-policy{stack}"
        runs.append((stack, role, policy_name))
    if prod:
        stack = os.environ["PROD_INFRA_STACK_NAME"]
        role = os.environ["PROD_PIPELINE_EXECUTION_ROLE"]
        policy_name = f"{APP_NAME}-pipeline-policy{stack}"
        runs.append((stack, role, policy_name))

    if aws_profile:
        os.environ["AWS_PROFILE"] = aws_profile

    # Policy document - define your permissions here
    policy_document = {"Version": "2012-10-17", "Statement": [{"Effect": "Allow", "Action": "*", "Resource": "*"}]}

    iam = boto3.client("iam")
    for stack, role, policy_name in runs:
        print(f"Stack: {stack}")
        print(f"Role: {role}")
        print(f"Policy name: {policy_name}")

        create_or_update_policy(policy_name, policy_document, iam, verbose, dry_run)
        attach_policy(role, policy_name, iam, verbose, dry_run)


if __name__ == "__main__":
    main()
